@@ -2,6 +2,7 @@
 
 #include <hyprtoolkit/core/Backend.hpp>
 #include <hyprutils/os/FileDescriptor.hpp>
+#include <hyprutils/cli/Logger.hpp>
 #include <hyprgraphics/resource/AsyncResourceGatherer.hpp>
 
 #include "../helpers/Env.hpp"
@@ -23,10 +24,12 @@ namespace Hyprtoolkit {
         virtual void                   addFd(int fd, std::function<void()>&& callback);
         virtual void                   removeFd(int fd);
         virtual SP<ISystemIconFactory> systemIcons();
-        virtual ASP<CTimer>  addTimer(const std::chrono::system_clock::duration& timeout, std::function<void(ASP<CTimer> self, void* data)> cb_, void* data, bool force = false);
-        virtual void         addIdle(const std::function<void()>& fn);
-        virtual void         enterLoop();
-        virtual SP<CPalette> getPalette();
+        virtual ASP<CTimer> addTimer(const std::chrono::system_clock::duration& timeout, std::function<void(ASP<CTimer> self, void* data)> cb_, void* data, bool force = false);
+        virtual void        addIdle(const std::function<void()>& fn);
+        virtual void        enterLoop();
+        virtual std::vector<SP<IOutput>>                                getOutputs();
+        virtual SP<CPalette>                                            getPalette();
+        virtual std::expected<SP<ISessionLockState>, eSessionLockError> aquireSessionLock();
 
         // ======================= Internal fns ======================= //
 
@@ -46,8 +49,6 @@ namespace Hyprtoolkit {
 
         Hyprutils::Memory::CSharedPointer<Aquamarine::CBackend> m_aqBackend;
 
-        LogFn                                                   m_logFn;
-
         bool                                                    m_terminate         = false;
         bool                                                    m_needsConfigReload = false;
 
