@@ -7,6 +7,8 @@
 #include "../window/ToolkitWindow.hpp"
 #include "../layout/Positioner.hpp"
 
+#include <algorithm>
+
 using namespace Hyprtoolkit;
 using namespace Hyprutils::Math;
 
@@ -74,6 +76,11 @@ void IElement::setAbsolutePosition(const Hyprutils::Math::Vector2D& offset) {
         impl->window->scheduleReposition(impl->self);
 }
 
+void IElement::setTooltip(std::string&& x) {
+    impl->tooltip    = std::move(x);
+    impl->hasTooltip = !impl->tooltip.empty();
+}
+
 std::optional<Hyprutils::Math::Vector2D> IElement::preferredSize(const Hyprutils::Math::Vector2D& parent) {
     return std::nullopt;
 }
@@ -132,7 +139,7 @@ void IElement::clearChildren() {
 }
 
 bool IElement::acceptsMouseInput() {
-    return impl->userRequestedMouseInput;
+    return impl->userRequestedMouseInput || impl->hasTooltip;
 }
 
 ePointerShape IElement::pointerShape() {
@@ -216,6 +223,10 @@ CBox IElement::opaqueBox() {
     return {};
 }
 
+void IElement::forceReposition() {
+    g_positioner->repositionNeeded(impl->self.lock(), true);
+}
+
 void SElementInternalData::setPosition(const CBox& box) {
     position = box;
     if (margin > 0)
