@@ -7,7 +7,9 @@ namespace Hyprtoolkit {
         std::string                path;
         float                      a        = 1.F;
         int                        rounding = 0;
+        bool                       sync     = false;
         SP<ISystemIconDescription> icon;
+        eImageFitMode              fitMode = IMAGE_FIT_MODE_STRETCH;
         CDynamicSize               size{CDynamicSize::HT_SIZE_PERCENT, CDynamicSize::HT_SIZE_PERCENT, {1, 1}};
     };
 
@@ -28,5 +30,6 @@ namespace Hyprtoolkit {
         std::string                                                           lastPath = "";
 
         Hyprutils::Math::Vector2D                                             preferredSvgSize();
+        void                                                                  postImageLoad();
     };
 }
